/*==================================================================*/
/*  TimeStamp - update the Last updated: line in a webpage          */
/*                                                                  */
/*  Note:  for DOS, you must link with  (W)setargv.obj, since       */
/*          wildcard expansion is not handled by the shell          */
/*                                                                  *
Last updated:  Sep 24, 2001 - 17:54 PDT
/*==================================================================*/


#include <iostream.h>
#include <fstream.h>
#include <time.h>
#include <stdio.h>
#include <string.h>


bool bVerbose = false;


/*==================================================================*/


int main (int argc, char *argv[])
{
    void TimeStamp (char *sFileName, char *sDateTime);
    void GetDateTime (char sDateTime[], const int nLength);

    char sDateTime [128];
    int  nArgc = 1;

    if (argc <= 1)
    {
        cout << "Usage: TimeStamp [-v] <file>..." << endl;
        return 0;
    }

    GetDateTime (sDateTime, sizeof sDateTime);

    if (!strcmp (argv[1], "-v") || !strcmp (argv[1], "-V"))
    {
        bVerbose = true;
        ++nArgc;
    }

    if (bVerbose)
        cout << "Timestamp:  " << sDateTime << endl << endl;

    for (int i = nArgc; i < argc; i++)
        TimeStamp (argv[i], sDateTime);

    if (bVerbose)
        cout << endl;

    return 0;
}


/*==================================================================*/


void TimeStamp (char *sFileName, char *sDateTime)
{
    char *TmpName (char *sName);
    char *sTmpName;
    char sLastUpdated [] = "Last updated:";
    bool bUpdated = false;

    if (bVerbose)
        cout << sFileName << "  ";

    if ((sTmpName = TmpName (sFileName)) == NULL)
    {
        cerr << endl
             << "Cannot create a temporary filename; " << sFileName << " not updated" << endl;
        return;
    }

    ifstream ins;
    ins.open (sFileName, ios::in | ios::nocreate | ios::binary);
    if (! ins)
    {
        cerr << endl
             << "Cannot open " << sFileName << endl << endl;
        return;
    }

    ofstream outs;
    outs.open (sTmpName, ios::out | ios::binary);
    if (! outs)
    {
        cerr << endl
             << "Cannot open temporary file " << sTmpName << endl << endl;
        ins.close();
        return;
    }

    char sBuffer [8192];
    while (*sBuffer = '\0', ins.getline (sBuffer, sizeof sBuffer), ins.gcount() > 0)
    {
        if (strncmp (sLastUpdated, sBuffer, sizeof sLastUpdated - 1))
            outs << sBuffer << endl;
        else
        {
            outs << sDateTime;
#ifdef WIN32
            outs << "\r";
#endif
            outs << endl;
            bUpdated = true;
        }
    }

    outs.close ();
    ins.close ();

    if (bUpdated)
    {
        remove (sFileName);
        rename (sTmpName, sFileName);
    }
    else
        remove (sTmpName);

    if (bVerbose)
    {
        if (bUpdated)
            cout << "timestamped";
        cout << endl;
    }
}


/*==================================================================*/


void GetDateTime (char sDateTime[], const int nLength)
{
    time_t ltime;
    struct tm *today;

    time (&ltime);
    today = localtime (&ltime);

    strftime (sDateTime, nLength,
        "Last updated:  %b %d, %Y - %H:%M %Z", today);
}


/*==================================================================*/


char *TmpName (char *sName)
{
    static char sTmpName [4096];

    strcpy (sTmpName, sName);
    strcat (sTmpName, "_TMP_");

    return sTmpName;
}


/*==================================================================*/
/*==================================================================*/


/* Testing...

Last updated:  Sep 24, 2001 - 17:54 PDT

*/

